package cn.com.qmth.scancentral.controller;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.api.utils.RequestUtil;
import com.qmth.boot.core.exception.ParameterException;

import cn.com.qmth.scancentral.bean.User;
import cn.com.qmth.scancentral.config.SysProperty;
import cn.com.qmth.scancentral.entity.AnswerCardEntity;
import cn.com.qmth.scancentral.entity.ExamEntity;
import cn.com.qmth.scancentral.enums.Role;
import cn.com.qmth.scancentral.enums.SystemMode;
import cn.com.qmth.scancentral.exception.ParameterExceptions;
import cn.com.qmth.scancentral.service.AdapteFileService;
import cn.com.qmth.scancentral.service.AnswerCardService;
import cn.com.qmth.scancentral.service.AnswerCardSubjectService;
import cn.com.qmth.scancentral.service.ExamService;
import cn.com.qmth.scancentral.service.SessionService;
import cn.com.qmth.scancentral.vo.ActiveTimeVo;
import cn.com.qmth.scancentral.vo.AnswerCardVo;
import cn.com.qmth.scancentral.vo.LogoutTimeVo;
import cn.com.qmth.scancentral.vo.apistatus.ApiStatusVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@Api(tags = "系统接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX)
@Aac(strict = false, auth = true)
public class SystemController extends BaseController {

    @Autowired
    private SessionService sessionService;

    @Autowired
    private SysProperty sysProperty;

    @Autowired
    private AnswerCardService answerCardService;

    @Autowired
    private ExamService examService;

    @Autowired
    private AdapteFileService adapteFileService;
    
    @Autowired
    private AnswerCardSubjectService answerCardSubjectService;

    @ApiOperation(value = "心跳接口")
    @PostMapping("heartbeat")
    public ActiveTimeVo heartbeat() {
        User user = getAccessUser();
        ActiveTimeVo vo = new ActiveTimeVo();
        vo.setActiveTime(sessionService.updateUserSession(user));
        return vo;
    }

    @ApiOperation(value = "登出接口")
    @PostMapping("logout")
    public LogoutTimeVo logout() {
        User user = getAccessUser();
        sessionService.userLogout(user);
        LogoutTimeVo vo = new LogoutTimeVo();
        vo.setLogoutTime(System.currentTimeMillis());
        return vo;
    }

    @Aac(auth = false)
    @ApiOperation(value = "服务端状态")
    @RequestMapping("status")
    public ApiStatusVo status(HttpServletRequest request) {
        ApiStatusVo vo = new ApiStatusVo();
        vo.setFileUriPrefix(sysProperty.getFileUriPrefix());
        vo.setVersion(sysProperty.getVersion());
        vo.setTime(System.currentTimeMillis());
        vo.setClientIP(RequestUtil.getIpAddress(request));
        vo.setSystemMode(SystemMode.current());
        return vo;
    }

    @ApiOperation(value = "答题卡卡格式查询")
    @PostMapping("/card/answer")
    public AnswerCardVo cardAnswer(HttpServletRequest request, @RequestParam Long examId,
            @RequestParam Integer number) {
        ExamEntity exam = examService.getById(examId);
        if (exam == null || !exam.getEnable()) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        AnswerCardEntity card = answerCardService.findByExamAndNumber(examId, number);
        if (card == null) {
            throw new ParameterException("找不到卡格式");
        }
        AnswerCardVo vo = new AnswerCardVo(card);
        User user = getAccessUser();
        if (user.getRole().equals(Role.SCANNER)) {
            vo.setAdapteFile(adapteFileService
                    .listByExamIdAndCardNumberForScanner(examId, number, RequestUtil.getIpAddress(request)));
        } else {
            vo.setAdapteFile(adapteFileService.listByExamIdAndCardNumber(examId, card.getNumber()));
        }
        vo.setSubjectList(answerCardSubjectService.findByExamAndNumber(examId, card.getNumber()));
        return vo;
    }
}