package cn.com.qmth.scancentral.controller.admin;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.task.AsyncTaskExecutor;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.concurrent.model.Semaphore;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.boot.core.exception.ReentrantException;

import cn.com.qmth.scancentral.bean.answergroup.StudentAnswerGroupCondition;
import cn.com.qmth.scancentral.bean.answergroup.StudnetAnswerGroupCountVo;
import cn.com.qmth.scancentral.bean.answergroup.StudnetAnswerGroupIdVo;
import cn.com.qmth.scancentral.bean.answergroup.StudnetAnswerGroupMarkedVo;
import cn.com.qmth.scancentral.bean.answergroup.StudnetAnswerGroupSummaryQuery;
import cn.com.qmth.scancentral.bean.answergroup.StudnetAnswerGroupSummaryVo;
import cn.com.qmth.scancentral.bean.answergroup.StudnetAnswerGroupVo;
import cn.com.qmth.scancentral.controller.BaseController;
import cn.com.qmth.scancentral.entity.StudentAnswerGroupEntity;
import cn.com.qmth.scancentral.enums.LockType;
import cn.com.qmth.scancentral.service.StudentAnswerGroupService;
import cn.com.qmth.scancentral.task.thread.StudentAnswerGroupBuildThread;
import cn.com.qmth.scancentral.task.thread.StudentAnswerGroupDeleteThread;
import cn.com.qmth.scancentral.task.thread.StudentAnswerGroupResetThread;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.sf.json.JSONObject;

@RestController
@Api(tags = "数据检查任务组接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/admin/check/student/answer/group")
@Aac(strict = false, auth = true)
public class StudentAnswerGroupController extends BaseController {

    @Autowired
    private StudentAnswerGroupService groupService;

    @Autowired
    private ConcurrentService concurrentService;

    @Autowired
    private AsyncTaskExecutor taskExecutor;

    @ApiOperation(value = "抽查任务列表")
    @PostMapping("list")
    public StudnetAnswerGroupVo list(@RequestParam Long examId) {
        return groupService.listByExamId(examId);
    }
    
    @ApiOperation(value = "抽查任务查询计数")
    @PostMapping("count")
    public StudnetAnswerGroupCountVo count(StudentAnswerGroupCondition domain) {
        return groupService.countByCondition(domain);
    }
    
    @ApiOperation(value = "抽查任务创建")
    @PostMapping("save")
    public StudnetAnswerGroupIdVo save(StudentAnswerGroupCondition domain) {
        return groupService.saveByCondition(domain);
    }


    @ApiOperation(value = "抽查任务组生成任务")
    @RequestMapping(value = "build", method = RequestMethod.POST)
    public JSONObject build(@RequestParam Long id) {
        StudentAnswerGroupEntity group = groupService.getById(id);
        if (group == null) {
            throw new ParameterException("分组不存在");
        }
        Semaphore semaphore = concurrentService.getSemaphore(LockType.STUDENT_ANSWER_GROUP_BUILD + "-" + id);
        if (semaphore.tryAcquire()) {
            taskExecutor.submit(new StudentAnswerGroupBuildThread(id, groupService, concurrentService, semaphore));
        } else {
            throw new ReentrantException("该分组数据操作繁忙，请稍后重试");
        }
        JSONObject result = new JSONObject();
        result.accumulate("id", id);
        result.accumulate("building", !semaphore.isAvailable());
        result.accumulate("updateTime", System.currentTimeMillis());
        return result;
    }

    @ApiOperation(value = "重置抽查任务组")
    @RequestMapping(value = "reset", method = RequestMethod.POST)
    public JSONObject reset(@RequestParam Long id) {
        StudentAnswerGroupEntity group = groupService.getById(id);
        if (group == null) {
            throw new ParameterException("分组不存在");
        }
        Semaphore semaphore = concurrentService.getSemaphore(LockType.STUDENT_ANSWER_GROUP_RESET + "-" + id);
        if (semaphore.tryAcquire()) {
            taskExecutor.submit(new StudentAnswerGroupResetThread(id, groupService, concurrentService, semaphore));
        } else {
            throw new ReentrantException("该分组数据操作繁忙，请稍后重试");
        }
        JSONObject result = new JSONObject();
        result.accumulate("id", id);
        return result;
    }

    @ApiOperation(value = "删除抽查任务组")
    @RequestMapping(value = "delete", method = RequestMethod.POST)
    public JSONObject delete(@RequestParam Long id) {
        StudentAnswerGroupEntity group = groupService.getById(id);
        if (group == null) {
            throw new ParameterException("分组不存在");
        }
        Semaphore semaphore = concurrentService.getSemaphore(LockType.STUDENT_ANSWER_GROUP_DELETE + "-" + id);
        if (semaphore.tryAcquire()) {
            taskExecutor.submit(new StudentAnswerGroupDeleteThread(id, groupService, concurrentService, semaphore));
        } else {
            throw new ReentrantException("该分组数据操作繁忙，请稍后重试");
        }
        JSONObject result = new JSONObject();
        result.accumulate("success", true);
        return result;
    }
    
    @ApiOperation(value = "获取异常题卡概要")
    @PostMapping("summary")
    public StudnetAnswerGroupSummaryVo summary(StudnetAnswerGroupSummaryQuery query) {
        return groupService.summary(query);
    }
    
    @ApiOperation(value = "获取标记异常考生列表")
    @PostMapping("marked")
    public List<StudnetAnswerGroupMarkedVo> marked(@RequestParam Long examId) {
        return groupService.marked(examId);
    }
}
