package cn.com.qmth.scancentral.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.boot.core.security.exception.AuthorizationException;

import cn.com.qmth.scancentral.bean.ExamConfigDomain;
import cn.com.qmth.scancentral.bean.ImportExamDomain;
import cn.com.qmth.scancentral.bean.User;
import cn.com.qmth.scancentral.dao.ExamDao;
import cn.com.qmth.scancentral.entity.ExamEntity;
import cn.com.qmth.scancentral.entity.ExamSummaryEntity;
import cn.com.qmth.scancentral.entity.SystemConfigEntity;
import cn.com.qmth.scancentral.enums.*;
import cn.com.qmth.scancentral.exception.ParameterExceptions;
import cn.com.qmth.scancentral.service.*;
import cn.com.qmth.scancentral.util.PageUtil;
import cn.com.qmth.scancentral.vo.ExamConfigVo;
import cn.com.qmth.scancentral.vo.ExamVo;
import cn.com.qmth.scancentral.vo.checkimage.RatioVo;
import cn.com.qmth.scancentral.vo.examinfo.ExamInfoVo;
import cn.com.qmth.scancentral.vo.examinfo.ExamQuery;
import cn.com.qmth.scancentral.vo.scanexaminfo.ScanExamInfoVo;
import cn.com.qmth.scancentral.vo.scanexamlist.ScanExamListQuery;
import cn.com.qmth.scancentral.vo.scanexamlist.ScanExamListVo;

@Service
public class ExamServiceImpl extends ServiceImpl<ExamDao, ExamEntity> implements ExamService {

    @Autowired
    private SessionService sessionService;

    @Autowired
    private AnswerCardService answerCardService;

    @Autowired
    private PackageCardService packageCardService;

    @Autowired
    private OmrTaskService omrTaskService;

    @Autowired
    private PackageTaskService packageTaskService;

    @Autowired
    private StudentService studentService;

    @Autowired
    private SubjectService subjectService;

    @Autowired
    private SystemConfigService systemConfigService;

    @Autowired
    private BatchService batchService;

    @Autowired
    private ConcurrentService concurrentService;

    @Autowired
    private OmrGroupService omrGroupService;

    @Autowired
    private UserService userService;

    @Autowired
    private QuestionService questionService;

    @Autowired
    private ExamSummaryService examSummaryService;

    @Autowired
    private StudentAnswerTaskService studentAnswerTaskService;

    @Override
    public PageResult<ExamVo> pageQuery(ExamQuery query, User user) {
        if (query.getEnable() == null) {
            throw new ParameterException("是否启用不能为空");
        }
        IPage<ExamVo> iPage = this.baseMapper
                .pageExam(new Page<>(query.getPageNumber(), query.getPageSize()), query, user.getSchoolId());
        return PageUtil.of(iPage);
    }

    @Override
    public List<ExamEntity> listEnable() {
        return list(new LambdaQueryWrapper<ExamEntity>().eq(ExamEntity::getEnable, true));
    }

    @Override
    public ExamInfoVo getExamInfoVo(Long examId, User accessUser) {
        ExamEntity exam = this.getById(examId);
        if (exam == null) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        if (!exam.getSchoolId().equals(accessUser.getSchoolId())) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        //ScanAnswerInfoVo info = studentService.scanAnswerInfo(examId);
        ExamInfoVo vo = new ExamInfoVo();
        ExamSummaryEntity es = examSummaryService.find(examId);
        vo.setId(examId);
        vo.setName(exam.getName());
        vo.setSchoolName(exam.getSchoolName());
        vo.getExamConfig().setAllowUnexistPaper(exam.getAllowUnexistPaper());
        vo.getExamConfig().setAnswerFrontCardType(exam.getAnswerFrontCardType());
        vo.getExamConfig().setImageTransferMode(exam.getImageTransferMode());
        vo.getExamConfig().setEnableSinglePageAnswer(exam.getEnableSinglePageAnswer());
        vo.getExamConfig().setMode(exam.getMode());
        vo.getExamConfig().setScanByPackage(exam.getScanByPackage());
        vo.getExamConfig().setEnableSyncVerify(exam.getEnableSyncVerify());
        vo.getExamConfig().setPaperTypeBarcodeContent(exam.getPaperTypeBarcodeContent());
        vo.getExamConfig().setAbsentBarcodeContent(exam.getAbsentBarcodeContent());

        vo.setSubjectConfig(subjectService.listConfigByExamId(examId));
        vo.getUpload().setEnable(exam.getEnableUpload());
        vo.getUpload().setProgress(es.getUploadProgress());

        vo.getExamData().setAnswerCard(answerCardService.getCountByExam(examId));
        vo.getExamData().setPackageCard(packageCardService.getCountByExam(examId));
        vo.getExamData().setProgress(0d);
        vo.getExamData().setStudentCount(es.getStudentCount());
        vo.getExamData().setSubjectCount(es.getSubjectCount());
        vo.getExamData().setSynching(concurrentService.isLocked(LockType.EXAM_SYNC + "-" + examId));
        vo.getExamData().setUpdateTime(exam.getDataSyncTime());

        SystemConfigEntity config = systemConfigService.find();
        boolean enableLogin = true;
        if (config != null && config.getScannerEnableLogin() != null) {
            enableLogin = config.getScannerEnableLogin();
        }
        vo.getScanner().setEnableLogin(enableLogin);
        vo.getScanner().setOnlineCount(sessionService.getOnlineScannerCount());
        vo.getScanner().setPassword(config == null ? null : config.getScannerPassword());

        vo.getAnswerScan().setAbsentSuspectCount(es.getAnswerAbsentSuspectCount());
        vo.getAnswerScan().setAssignedCount(es.getAnswerAssignedCount());
        vo.getAnswerScan().setOmrAbsentCount(es.getAnswerOmrAbsentCount());
        vo.getAnswerScan().setBatchCount(es.getAnswerBatchCount());
        vo.getAnswerScan().setIncompleteCount(es.getAnswerIncompleteCount());
        vo.getAnswerScan().setManualAbsentCount(es.getAnswerManualAbsentCount());
        vo.getAnswerScan().setScannedCount(es.getAnswerScannedCount());
        vo.getAnswerScan().setTotalCount(es.getStudentCount());
        vo.getAnswerScan().setUnexistCount(es.getAnswerUnexistCount());
        vo.getAnswerScan().setMismatchCount(es.getAnswerMismatchCount());
        vo.getAnswerScan().setStudentAnswerCount(es.getStudentAnswerCount());

        vo.getPackageScan().setScannedCount(es.getPackageScannedCount());
        vo.getPackageScan().setTotalCount(es.getPackageTotalCount());

        vo.getVerifyTask().setTodoCount(batchService.getVerifyCount(examId));
        int omrTaskCount = omrTaskService.getCountByExam(examId);
        int omrTaskTodoCount = omrTaskService.getCountByExamAndStatus(examId, TaskStatus.WAITING);
        vo.getOmrTask().setTotalCount(omrTaskCount);
        vo.getOmrTask().setTodoCount(omrTaskTodoCount >= omrTaskCount ? omrTaskCount : omrTaskTodoCount);
        vo.getOmrTask().setUnarbitrateCount(omrTaskService.getCountByExamAndStatus(examId, TaskStatus.WAIT_ARBITRATE));

        vo.getPackageTask().setTodoCount(
                packageTaskService.getCountByExamAndStatus(examId, TaskStatus.WAITING, TaskStatus.WAIT_ARBITRATE));
        vo.getPackageTask().setTotalCount(packageTaskService.getCountByExam(examId));

        vo.getImageCheckTask().setCheckRatio(exam.getImageCheckRatio());
        vo.getImageCheckTask()
                .setFinishCount(batchService.getCheckCountByExamId(examId, CheckStatus.FINISH, accessUser));
        vo.getImageCheckTask()
                .setTodoCount(batchService.getCheckCountByExamId(examId, CheckStatus.WAITING, accessUser));

        vo.getAuditor().setOnlineCount(sessionService.getOnlineAuditorCount());
        vo.getAuditor().setUserCount(userService.countByRole(accessUser.getSchoolId(), Role.AUDITOR));

        vo.getStudentAnswerTask().setTodoCount(studentAnswerTaskService.getCountByExamAndStatus(examId, StudentAnswerTaskStatus.WAITING));
        vo.getStudentAnswerTask().setTotalCount(studentAnswerTaskService.getCountByExam(examId));

        vo.getExamStatusCheck().setExamNumberFillCount(exam.getExamNumberFillCount());

        if (accessUser.getRole().equals(Role.AUDITOR)) {
            vo.getAssignedCheck().setTodoCount(studentService.getCountByExamAndAssignedCheckCount(examId, 0));
        }
        if (accessUser.getRole().equals(Role.SCHOOL_ADMIN) || accessUser.getRole().equals(Role.SCAN_ADMIN)) {
            vo.getAssignedCheck().setTodoCount(studentService.getCountByExamAndAssignedCheckCount(examId, 1));
        }
        return vo;
    }

    @Override
    public ScanExamInfoVo getScanExamInfo(Long id, User accessUser) {
        ExamEntity exam = this.getById(id);
        if (exam == null || (exam.getEnable() != null && !exam.getEnable())) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        ExamSummaryEntity es = examSummaryService.find(exam.getId());
        ScanExamInfoVo vo = new ScanExamInfoVo();
        vo.setId(id);
        vo.setName(exam.getName());
        vo.setSchoolName(exam.getSchoolName());
        vo.getExamConfig().setAllowUnexistPaper(exam.getAllowUnexistPaper());
        vo.getExamConfig().setAnswerFrontCardType(exam.getAnswerFrontCardType());
        vo.getExamConfig().setImageTransferMode(exam.getImageTransferMode());
        vo.getExamConfig().setEnableSinglePageAnswer(exam.getEnableSinglePageAnswer());
        vo.getExamConfig().setMode(exam.getMode());
        vo.getExamConfig().setScanByPackage(exam.getScanByPackage());
        vo.getExamConfig().setEnableSyncVerify(exam.getEnableSyncVerify());
        vo.getExamConfig().setAbsentBarcodeContent(exam.getAbsentBarcodeContent());
        vo.getExamConfig().setPaperTypeBarcodeContent(exam.getPaperTypeBarcodeContent());

        vo.getAnswerScan().setCardCount(answerCardService.getCountByExam(id));
        vo.getAnswerScan().setTotalCount(es.getStudentCount());
        vo.getAnswerScan().setScannedCount(es.getAnswerScannedCount());

        vo.getPackageScan().setCardCount(packageCardService.getCountByExam(id));
        vo.getPackageScan().setTotalCount(es.getPackageTotalCount());
        vo.getPackageScan().setScannedCount(es.getPackageScannedCount());

        vo.getOmrTask().setTodoCount(omrTaskService.getCountByExamAndStatus(id, TaskStatus.WAITING));
        vo.getOmrTask().setFinishCount(omrTaskService.getCountByExamAndStatus(id, TaskStatus.PROCESSED));

        vo.getPackageTask().setTodoCount(packageTaskService.getCountByExamAndStatus(id, TaskStatus.WAITING));
        vo.getPackageTask().setFinishCount(packageTaskService.getCountByExamAndStatus(id, TaskStatus.PROCESSED));

        vo.setSubjectConfig(subjectService.listConfigByExamId(id));

        vo.getStudentAnswerTask().setTodoCount(studentAnswerTaskService.getCountByExamAndStatus(id, StudentAnswerTaskStatus.WAITING));
        vo.getStudentAnswerTask().setFinishCount(studentAnswerTaskService.getCountByExamAndStatus(id, StudentAnswerTaskStatus.PROCESSED));

        return vo;
    }

    @Override
    public List<ScanExamListVo> getScanExamList() {
        ScanExamListQuery query = new ScanExamListQuery();
        query.setEnable(true);
        return this.baseMapper.getExamList(query);
    }

    @Transactional
    @Override
    public ExamConfigVo config(User user, ExamConfigDomain domain) {
        if (!Role.SCHOOL_ADMIN.equals(user.getRole())) {
            throw AuthorizationException.NO_PERMISSION;
        }
        ExamEntity exam = this.getById(domain.getId());
        if (exam == null) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        if (!exam.getSchoolId().equals(user.getSchoolId())) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
            if (domain.getAllowUnexistPaper() != null) {
                exam.setAllowUnexistPaper(domain.getAllowUnexistPaper());
            }
            if (domain.getEnableSinglePageAnswer() != null) {
                exam.setEnableSinglePageAnswer(domain.getEnableSinglePageAnswer());
            }
            if (domain.getEnableSyncVerify() != null) {
                exam.setEnableSyncVerify(domain.getEnableSyncVerify());
            }
            if (domain.getScanByPackage() != null) {
                exam.setScanByPackage(domain.getScanByPackage());
            }
            if (domain.getAbsentBarcodeContent() != null) {
                exam.setAbsentBarcodeContent(domain.getAbsentBarcodeContent());
            }
            if (domain.getAnswerFrontCardType() != null) {
                exam.setAnswerFrontCardType(domain.getAnswerFrontCardType());
            }
            if (domain.getImageTransferMode() != null) {
                exam.setImageTransferMode(domain.getImageTransferMode());
            }
            if (domain.getMode() != null) {
                exam.setMode(domain.getMode());
            }
            if (domain.getPaperTypeBarcodeContent() != null) {
                exam.setPaperTypeBarcodeContent(domain.getPaperTypeBarcodeContent());
            }
            exam.setId(domain.getId());
            exam.setAbsentBarcodeContent(domain.getAbsentBarcodeContent());
            exam.setAllowUnexistPaper(domain.getAllowUnexistPaper());
            exam.setAnswerFrontCardType(domain.getAnswerFrontCardType());
            exam.setImageTransferMode(domain.getImageTransferMode());
            exam.setEnableSyncVerify(domain.getEnableSyncVerify());
            exam.setEnableSinglePageAnswer(domain.getEnableSinglePageAnswer());
            exam.setMode(domain.getMode());
            exam.setPaperTypeBarcodeContent(domain.getPaperTypeBarcodeContent());
            exam.setScanByPackage(domain.getScanByPackage());
            if (exam.getEnableSyncVerify() == false) {
                batchService.batchVerifyCancel(user, exam.getId());
            }
            this.saveOrUpdate(exam);
            return new ExamConfigVo(exam);
    }

    @Transactional
    @Override
    public ExamEntity save(ImportExamDomain domain) {
        if (SystemMode.MARKINGCLOUD.equals(SystemMode.current())) {
            throw new ParameterException("非本地模式不支持此功能");
        }
        if (domain.getAllowUnexistPaper() == null) {
            throw new ParameterException("AllowUnexistPaper 不能为空");
        }
        if (domain.getEnableSinglePageAnswer() == null) {
            throw new ParameterException("EnableSinglePageAnswer 不能为空");
        }
        if (domain.getEnableSyncVerify() == null) {
            throw new ParameterException("EnableSyncVerify 不能为空");
        }
        if (domain.getScanByPackage() == null) {
            throw new ParameterException("ScanByPackage 不能为空");
        }
        if (domain.getAbsentBarcodeContent() == null) {
            throw new ParameterException("AbsentBarcodeContent 不能为空");
        }
        if (domain.getAnswerFrontCardType() == null) {
            throw new ParameterException("AnswerFrontCardType 不能为空");
        }
        if (domain.getImageTransferMode() == null) {
            domain.setImageTransferMode(ImageTransferMode.OFF);
            // throw new ParameterException("ImageTransferMode 不能为空");
        }
        if (domain.getMode() == null) {
            throw new ParameterException("ExamMode 不能为空");
        }
        if (domain.getPaperTypeBarcodeContent() == null) {
            throw new ParameterException("PaperTypeBarcodeContent 不能为空");
        }
        if (domain.getExamNumberFillCount() == null) {
            throw new ParameterException("ExamNumberFillCount 不能为空");
        }
        ExamEntity exam = new ExamEntity();
        exam.setId(domain.getId());
        exam.setSchoolId(domain.getSchoolId());
        exam.setSchoolName(domain.getSchoolName());
        exam.setName(domain.getName());
        exam.setEnable(true);
        exam.setEnableUpload(false);
        exam.setImageCheckRatio(0d);
        exam.setAllowUnexistPaper(domain.getAllowUnexistPaper());
        exam.setEnableSinglePageAnswer(domain.getEnableSinglePageAnswer());
        exam.setEnableSyncVerify(domain.getEnableSyncVerify());
        exam.setScanByPackage(domain.getScanByPackage());
        exam.setAbsentBarcodeContent(domain.getAbsentBarcodeContent());
        exam.setAnswerFrontCardType(domain.getAnswerFrontCardType());
        exam.setImageTransferMode(domain.getImageTransferMode());
        exam.setMode(domain.getMode());
        exam.setPaperTypeBarcodeContent(domain.getPaperTypeBarcodeContent());
        if (getById(domain.getId()) == null) {
            omrGroupService.addFixOmrCondition(1L, exam.getId());
        }
        exam.setExamNumberFillCount(domain.getExamNumberFillCount());
        this.saveOrUpdate(exam);
        return exam;
    }

    @Override
    public RatioVo updateRatio(Long examId, Double ratio) {
        if (ratio == null || ratio < 0 || ratio > 1) {
            throw new ParameterException("ratio 不符合");
        }
        ExamEntity exam = this.getById(examId);
        if (exam == null) {
            throw new ParameterException("Exam 不能为空");
        }
        exam.setImageCheckRatio(ratio);
        this.updateById(exam);
        return new RatioVo(ratio);
    }

    @Transactional
    @Override
    public void updateDataSyncTime(Long examId) {
        ExamEntity exam = this.getById(examId);
        if (exam == null) {
            throw new ParameterException("Exam 不能为空");
        }
        exam.setDataSyncTime(System.currentTimeMillis());
        this.updateById(exam);

    }

    @Transactional
    @Override
    public void updateCardSyncTime(Long examId) {
        ExamEntity exam = this.getById(examId);
        if (exam == null) {
            throw new ParameterException("Exam 不能为空");
        }
        exam.setCardSyncTime(System.currentTimeMillis());
        this.updateById(exam);

    }

    @Override
    public void updateExamNumberFillCount(Long examId, Integer examNumberFillCount) {
        ExamEntity exam = this.getById(examId);
        if (exam == null) {
            throw new ParameterException("Exam 不能为空");
        }
        exam.setExamNumberFillCount(examNumberFillCount);
        this.updateById(exam);
    }

    @Override
    @Transactional
    public ExamEntity updateEnableSyncVerify(User user, Long examId, Boolean enableSyncVerify) {
        ExamEntity exam = this.getById(examId);
        if (exam == null) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }
        if (enableSyncVerify == false) {
            batchService.batchVerifyCancel(user, examId);
        }
        exam.setEnableSyncVerify(enableSyncVerify);
        this.saveOrUpdate(exam);
        return exam;
    }
}
