package cn.com.qmth.scancentral.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.com.qmth.scancentral.bean.User;
import cn.com.qmth.scancentral.bean.answertask.StudentAnswerTaskPageVo;
import cn.com.qmth.scancentral.bean.answertask.StudentAnswerTaskPaperVo;
import cn.com.qmth.scancentral.bean.answertask.StudentAnswerTaskSaveDomain;
import cn.com.qmth.scancentral.bean.answertask.StudentAnswerTaskSaveVo;
import cn.com.qmth.scancentral.bean.answertask.StudentAnswerTaskStatusVo;
import cn.com.qmth.scancentral.bean.answertask.StudentAnswerTaskVo;
import cn.com.qmth.scancentral.dao.StudentAnswerTaskDao;
import cn.com.qmth.scancentral.entity.PaperEntity;
import cn.com.qmth.scancentral.entity.PaperPageEntity;
import cn.com.qmth.scancentral.entity.StudentAnswerGroupEntity;
import cn.com.qmth.scancentral.entity.StudentAnswerTaskEntity;
import cn.com.qmth.scancentral.entity.StudentEntity;
import cn.com.qmth.scancentral.entity.StudentPaperEntity;
import cn.com.qmth.scancentral.entity.SubjectEntity;
import cn.com.qmth.scancentral.enums.PaperTypeStatus;
import cn.com.qmth.scancentral.enums.StudentAnswerTaskStatus;
import cn.com.qmth.scancentral.enums.TaskStatus;
import cn.com.qmth.scancentral.exception.NotFoundExceptions;
import cn.com.qmth.scancentral.service.PaperPageService;
import cn.com.qmth.scancentral.service.PaperService;
import cn.com.qmth.scancentral.service.StudentAnswerGroupService;
import cn.com.qmth.scancentral.service.StudentAnswerTaskService;
import cn.com.qmth.scancentral.service.StudentPaperService;
import cn.com.qmth.scancentral.service.StudentService;
import cn.com.qmth.scancentral.service.SubjectService;
import cn.com.qmth.scancentral.support.TaskLock;
import cn.com.qmth.scancentral.support.TaskLockUtil;

@Service
public class StudentAnswerTaskServiceImpl extends ServiceImpl<StudentAnswerTaskDao, StudentAnswerTaskEntity>
        implements StudentAnswerTaskService {

    @Autowired
    private PaperService paperService;

    @Autowired
    private StudentAnswerGroupService groupService;

    @Autowired
    private PaperPageService pageService;

    @Autowired
    private StudentService studentService;

    @Autowired
    private SubjectService subjectService;

    @Autowired
    private StudentPaperService studentPaperService;

    @Override
    public int getCountByExamAndStatus(Long examId, StudentAnswerTaskStatus status) {
        QueryWrapper<StudentAnswerTaskEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentAnswerTaskEntity> lw = wrapper.lambda();
        lw.eq(StudentAnswerTaskEntity::getStatus, status);
        lw.eq(StudentAnswerTaskEntity::getExamId, examId);
        return this.count(wrapper);
    }

    @Override
    public int getCountByExamAndStatusAndDevice(Long examId, String device, StudentAnswerTaskStatus status) {
        QueryWrapper<StudentAnswerTaskEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentAnswerTaskEntity> lw = wrapper.lambda();
        lw.eq(StudentAnswerTaskEntity::getStatus, status);
        lw.eq(StudentAnswerTaskEntity::getDevice, device);
        lw.eq(StudentAnswerTaskEntity::getExamId, examId);
        return this.count(wrapper);
    }

    @Override
    public int getCountByExamAndGroupAndStatus(Long examId, Long groupId, StudentAnswerTaskStatus status) {
        QueryWrapper<StudentAnswerTaskEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentAnswerTaskEntity> lw = wrapper.lambda();
        lw.eq(StudentAnswerTaskEntity::getStatus, status);
        lw.eq(StudentAnswerTaskEntity::getExamId, examId);
        lw.eq(StudentAnswerTaskEntity::getGroupId, groupId);
        return this.count(wrapper);
    }

    @Transactional
    @Override
    public void deleteByGroupId(Long groupId) {
        UpdateWrapper<StudentAnswerTaskEntity> wrapper = new UpdateWrapper<>();
        LambdaUpdateWrapper<StudentAnswerTaskEntity> lw = wrapper.lambda();
        lw.eq(StudentAnswerTaskEntity::getGroupId, groupId);
        this.baseMapper.delete(wrapper);
    }

    @Transactional
    @Override
    public void resetByGroup(StudentAnswerGroupEntity group) {
        UpdateWrapper<StudentAnswerTaskEntity> wrapper = new UpdateWrapper<>();
        LambdaUpdateWrapper<StudentAnswerTaskEntity> lw = wrapper.lambda();
        lw.set(StudentAnswerTaskEntity::getStatus, TaskStatus.WAITING);
        lw.set(StudentAnswerTaskEntity::getDevice, null);
        lw.eq(StudentAnswerTaskEntity::getGroupId, group.getId());
        this.update(wrapper);
    }

    @Transactional
    @Override
    public void saveTask(StudentAnswerGroupEntity group, List<Long> studentIds) {
        List<StudentAnswerTaskEntity> list = new ArrayList<>();
        for (Long studentId : studentIds) {
            StudentAnswerTaskEntity task = new StudentAnswerTaskEntity();
            task.setExamId(group.getExamId());
            task.setGroupId(group.getId());
            task.setStatus(StudentAnswerTaskStatus.WAITING);
            task.setStudentId(studentId);
            list.add(task);
        }
        this.saveBatch(list);
        groupService.updateTotalCount(group.getId());
    }

    @Override
    public StudentAnswerTaskVo getTask(Long examId, String account) {
        return findWaitingTask(examId, account);
    }

    private StudentAnswerTaskVo findWaitingTask(Long examId, String account) {
        int retry = 0;
        StudentAnswerTaskVo task = null;
        while (task == null) {
            List<StudentAnswerTaskEntity> list = this.findUnMarked(examId, retry * 20, 20,
                    StudentAnswerTaskStatus.WAITING);
            if (list.isEmpty()) {
                break;
            }
            for (StudentAnswerTaskEntity t : list) {
                if (this.apply(t, account)) {
                    task = toTaskVo(t);
                    break;
                }
            }
            if (task == null) {
                retry++;
            }
        }
        if (task == null) {
            throw NotFoundExceptions.NO_STUDENT_ANSWER_TASK;
        }
        return task;
    }

    private StudentAnswerTaskVo toTaskVo(StudentAnswerTaskEntity task) {
        StudentAnswerTaskVo vo = new StudentAnswerTaskVo();
        StudentEntity student = studentService.getById(task.getStudentId());
        vo.setCardNumber(student.getCardNumber());
        vo.setExamNumber(student.getExamNumber());
        vo.setId(task.getId());
        vo.setName(student.getName());
        vo.setSubjectCode(student.getSubjectCode());
        vo.setOmrAbsent(student.getOmrAbsent());
        vo.setAssigned(student.getAssigned());
        vo.setCheckMark(student.getCheckMark());
        SubjectEntity subject = subjectService.findByExamIdAndCode(student.getExamId(), student.getSubjectCode());
        vo.setSubjectName(subject.getName());
        List<StudentAnswerTaskPaperVo> papers = new ArrayList<>();
        vo.setPapers(papers);
        List<StudentPaperEntity> spapers = studentPaperService.findByStudentId(student.getId());
        StudentAnswerGroupEntity group = groupService.getById(task.getGroupId());
        for (StudentPaperEntity sp : spapers) {
            PaperEntity paper = paperService.getById(sp.getPaperId());
            if (group.getConditions().getAssigned() != null && group.getConditions().getAssigned()
                    && paper.getAssigned() != null && !paper.getAssigned()) {
                continue;
            }
            boolean paperTypeSuccss = true;
            StudentAnswerTaskPaperVo paperVo = new StudentAnswerTaskPaperVo();
            paperVo.setNumber(paper.getNumber());
            paperVo.setAssigned(paper.getAssigned());
            List<PaperPageEntity> pages = pageService.listByPaperId(sp.getPaperId());
            if (group.getConditions().getPaperTypeStatus() != null) {
                paperTypeSuccss = paperTypeSuccss(pages.get(0), group.getConditions().getPaperTypeStatus());
            }
            for (PaperPageEntity page : pages) {
                StudentAnswerTaskPageVo pageVo = new StudentAnswerTaskPageVo();
                pageVo.setIndex(page.getPageIndex());
                pageVo.setSheetUri(page.getSheetPath());
                pageVo.setSliceUri(page.getSlicePath());
                paperVo.addPage(pageVo);
            }
            if (paperTypeSuccss) {
                papers.add(paperVo);
            }
        }
        return vo;
    }

    private boolean paperTypeSuccss(PaperPageEntity page, PaperTypeStatus paperTypeStatus) {
        if (PaperTypeStatus.OK.equals(paperTypeStatus)) {
            if (page.getPaperType() == null || page.getPaperType().getResult() == null
                    || "#".equals(page.getPaperType().getResult()) || "?".equals(page.getPaperType().getResult())) {
                return false;
            } else {
                return true;
            }
        } else if (PaperTypeStatus.ERROR.equals(paperTypeStatus)) {
            if (page.getPaperType() != null && page.getPaperType().getResult() != null
                    && "?".equals(page.getPaperType().getResult())) {
                return true;
            } else {
                return false;
            }
        } else if (PaperTypeStatus.BLANK.equals(paperTypeStatus)) {
            if (page.getPaperType() != null && page.getPaperType().getResult() != null
                    && "#".equals(page.getPaperType().getResult())) {
                return true;
            } else {
                return false;
            }
        }
        return false;
    }

    @Transactional
    @Override
    public StudentAnswerTaskSaveVo submitTask(StudentAnswerTaskSaveDomain domain, User user) {
        StudentAnswerTaskEntity task = this.getById(domain.getId());
        task.setStatus(StudentAnswerTaskStatus.PROCESSED);
        task.setDevice(user.getAccount());
        studentService.updateCheckMark(task.getStudentId(), domain.getTag());
        this.saveOrUpdate(task);
        this.releaseByTask(task);
        StudentAnswerTaskSaveVo vo = new StudentAnswerTaskSaveVo();
        vo.setId(domain.getId());
        vo.setStatus(getStatus(task.getExamId(), user));
        return vo;
    }

    @Override
    public StudentAnswerTaskStatusVo getStatus(Long examId, User user) {
        StudentAnswerTaskStatusVo vo = new StudentAnswerTaskStatusVo();
        vo.setFinishCount(
                this.getCountByExamAndStatusAndDevice(examId, user.getAccount(), StudentAnswerTaskStatus.PROCESSED));
        vo.setTodoCount(this.getCountByExamAndStatus(examId, StudentAnswerTaskStatus.WAITING));
        return vo;
    }

    private boolean apply(StudentAnswerTaskEntity t, String account) {
        TaskLock taskLock = TaskLockUtil.getStudentAnswerTask(t.getExamId().toString());
        boolean lock = taskLock.add(t.getId(), account);
        // 上锁失败直接返回
        if (!lock) {
            return false;
        }
        // 重复校验任务状态
        if (t.getStatus().equals(StudentAnswerTaskStatus.WAITING)) {
            return true;
        } else {
            taskLock.remove(t.getId());
            return false;
        }
    }

    @Override
    public void releaseByUser(Long examId, String account) {
        TaskLock taskLock = TaskLockUtil.getStudentAnswerTask(examId.toString());
        taskLock.clear(account);
    }

    @Override
    public void releaseByTask(StudentAnswerTaskEntity t) {
        TaskLock taskLock = TaskLockUtil.getStudentAnswerTask(t.getExamId().toString());
        taskLock.remove(t.getId());
    }

    @Override
    public boolean hasApplied(StudentAnswerTaskEntity t, String account) {
        TaskLock taskLock = TaskLockUtil.getStudentAnswerTask(t.getExamId().toString());
        return taskLock.exist(t.getId(), account);
    }

    @Override
    public List<StudentAnswerTaskEntity> findUnMarked(Long examId, int pageNumber, int pageSize,
            StudentAnswerTaskStatus status) {
        return this.baseMapper.findUnMarked(examId, pageNumber, pageSize, status);
    }

    @Override
    public int getCountByExam(Long examId) {
        QueryWrapper<StudentAnswerTaskEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentAnswerTaskEntity> lw = wrapper.lambda();
        lw.eq(StudentAnswerTaskEntity::getExamId, examId);
        return this.count(wrapper);
    }

    @Transactional
    @Override
    public void deleteByStudentId(Long examId, Long studentId) {
        QueryWrapper<StudentAnswerTaskEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentAnswerTaskEntity> lw = wrapper.lambda();
        lw.eq(StudentAnswerTaskEntity::getExamId, examId);
        lw.eq(StudentAnswerTaskEntity::getStudentId, studentId);
        this.baseMapper.delete(wrapper);
        groupService.updateTotalCountByExamId(examId);
    }

    @Override
    public StudentAnswerTaskVo history(Long examId, Long taskId, String account,Boolean next) {
        StudentAnswerTaskEntity cur = null;
        if (taskId != null) {
            cur = this.getById(taskId);
        }
        QueryWrapper<StudentAnswerTaskEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentAnswerTaskEntity> lw = wrapper.lambda();
        lw.eq(StudentAnswerTaskEntity::getExamId, examId);
        lw.eq(StudentAnswerTaskEntity::getDevice, account);
        lw.eq(StudentAnswerTaskEntity::getStatus, TaskStatus.PROCESSED);
        if(next!=null && next) {
            if (taskId != null) {
                lw.gt(StudentAnswerTaskEntity::getId, cur.getId());
            }
            lw.orderByAsc(StudentAnswerTaskEntity::getId);
        }else {
            if (taskId != null) {
                lw.lt(StudentAnswerTaskEntity::getId, cur.getId());
            }
            lw.orderByDesc(StudentAnswerTaskEntity::getId);
        }
        wrapper.last("LIMIT 1");
        StudentAnswerTaskEntity task = this.baseMapper.selectOne(wrapper);
        if (task == null) {
            throw NotFoundExceptions.NO_STUDENT_ANSWER_TASK;
        }
        return toTaskVo(task);
    }
}
